import { eq } from './Matchers';

export default class MockObject {
  
  static fromPrototype(prototype) {
    let isFunction = (property) => {
      return typeof property === 'function';
    }

    let functions = Object.getOwnPropertyNames(prototype)
      .filter((p) => (isFunction(prototype[p])))
      .filter((p) => (p !== 'constructor'));
    return MockObject.fromFunctionNames(...functions);
  }

  static fromFunctionNames(...functionNames) {
    let mock = {};
    functionNames.forEach((fn) => mock[fn] = MockObject.createMockFunction());
    return mock;
  }

  static createMockFunction() {
    let expectedCalls = new Map();
    let fn = jest.fn();
    fn.mockImplementation(MockObject._handleCall.bind(null, expectedCalls, fn.mock.calls));
    fn.mockReturnValueWhenCalledWith = MockObject._mockReturnValueWhenCallWith.bind(null, expectedCalls);

    return fn;
  }

  static _handleCall(expectedCalls, calls) {
    let lastCall = calls[calls.length - 1];
    let result;
    expectedCalls.forEach((value, key, map) => {
        if (lastCall.length === key.length) {
            let matches = true;
            for (let i = 0; i < key.length; i++) {
                let matcher = key[i];
                if (!(matcher(lastCall[i]))) {
                    matches = false;
                    break;
                }
            }
            if (matches) {
                result = value;
                return;
            }
        }
    });
    if (result) {
        return result;
    }
  }

  static _mockReturnValueWhenCallWith(expectedCalls, value, ...args) {
    let isFunction = (fn) => {
        return typeof fn === 'function';
    }
    
    let keys = [];
    args.forEach((a) => {
        if (!isFunction(a)) {
            keys.push(eq(a));
        } else {
            keys.push(a);
        }            
    })
    expectedCalls.set(keys, value);
  }
}